@@ -1,13 +1,17 @@
 /************************************************************************/
-/* GL_String.cc: Replacement for GNU's libg++ String class...      	*/
+/* GL_String.cc: Replacement for GNU's libg++ string class...      	*/
 /************************************************************************/
 /* Author: Andreas Gerstlauer <gerstl>		first version: 03/02/00 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 02/20/06 AG	changed internal string length from 'unsigned short' to 'size_t'
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 10/05/04 RD	adjustments for compilation on 64-bit architectures
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 04/19/02 AG	fixed a bug in string comparing, added regression test
  * 11/21/01 RD	took out default arguments from function definitions
@@ -23,17 +27,18 @@
 const char* const _nullString = "";
 
 
-// ------------------- String methods ---------------------
+// ------------------- GL_String methods ---------------------
 
 
 // format string
-String& String::vform(const char* format, va_list args)
+GL_String& GL_String::vform(const char* format, va_list args)
 {
   int dummy;
   char* old_str;
-  va_list arglst = args;
+  va_list arglst;
+  va_copy(arglst, args);
   const char* s = format;
-  int len = strlen(format);
+  size_t len = strlen(format);
   
   // Loop over format string to determine length needed
   while( (s = strchr(s, '%')) )
@@ -81,12 +86,13 @@ String& String::vform(const char* format, va_list args)
   
   // Assign string
   _len = vsprintf(_str, format, args);
+  assert(((int)_len) >= 0);
   assert(_len < _size);
   
   return *this;
 }
 
-String& String::form(const char* format, ...)
+GL_String& GL_String::form(const char* format, ...)
 {  
   va_list  arglst;  
   va_start(arglst, format);
@@ -96,8 +102,8 @@ String& String::form(const char* format, ...)
 }
 
 
-//  return number of occurences of target in String
-int String::freq(char c) const
+//  return number of occurences of target in string
+int GL_String::freq(char c) const
 {
    int   count = 0;
    char* cur = _str;
@@ -109,13 +115,13 @@ int String::freq(char c) const
    return count;   
 }
 
-int String::freq(const char* t) const
+int GL_String::freq(const char* t) const
 {
   int   count = 0;
   
   if (t) {
     char* cur = _str;
-    unsigned int t_len = strlen(t);
+    size_t t_len = strlen(t);
    
     while ((cur = strstr(cur, t))) {
       count++;
@@ -129,65 +135,90 @@ int String::freq(const char* t) const
 
 // Substring extraction
 
-SubString String::at(const char* t, int startpos /* = 0 */)
+GL_SubString GL_String::at(const char* t, size_t startpos /* = 0 */)
 {
   assert ((startpos >= 0) && (startpos <= _len));
   
   char* p = (_len && t)? strstr((startpos > 0)? &_str[startpos] : _str, t) : 0;
 
-  return p? at(p - _str, strlen(t)) : at(0, 0);
+  assert ((p - _str) >= 0);
+  return p? at((size_t)(p - _str), strlen(t)) : at((size_t)0, (size_t)0);
 }
 
-SubString String::at(char c, int startpos /* = 0 */) 
+GL_SubString GL_String::at(char c, size_t startpos /* = 0 */) 
 {
   assert ((startpos >= 0) && (startpos <= _len));
   
   char* p = _len? strchr((startpos > 0)? &_str[startpos] : _str, c) : 0;
 
-  return p? at(p - _str, 1) : at(0, 0);
+  assert ((p - _str) >= 0);
+  return p? at((size_t)(p - _str), 1) : at((size_t)0, (size_t)0);
 }
 
 
-SubString String::before(const char* t, int startpos /* = 0 */)
+GL_SubString GL_String::before(const char* t, size_t startpos /* = 0 */)
 {
   assert (startpos <= _len);
   
   char* p = (_len && t)? strstr((startpos > 0)? &_str[startpos] : _str, t) : 0;
 
-  return p? before(p - _str) : before(0);
+  assert ((p - _str) >= 0);
+  return p? before((size_t)(p - _str)) : before((size_t)0);
 }
 
-SubString String::before(char c, int startpos /* = 0 */) 
+GL_SubString GL_String::before(char c, size_t startpos /* = 0 */) 
 {
   assert (startpos <= _len);
   
   char* p = _len? strchr((startpos > 0)? &_str[startpos] : _str, c) : 0;
 
-  return p? before(p - _str) : before(0);
+  assert ((p - _str) >= 0);
+  return p? before((size_t)(p - _str)) : before((size_t)0);
 }
 
-SubString String::after(const char* t, int startpos /* = 0 */)
+GL_SubString GL_String::beforelast(char c, size_t startpos /* = 0 */) 
+{
+  assert (startpos <= _len);
+  
+  char* p = _len? strrchr((startpos > 0)? &_str[startpos] : _str, c) : 0;
+
+  assert ((p - _str) >= 0);
+  return p? before((size_t)(p - _str)) : before((size_t)0);
+}
+
+GL_SubString GL_String::after(const char* t, size_t startpos /* = 0 */)
 {
   assert (startpos <= _len);
   
   char* p = (_len && t)? strstr((startpos > 0)? &_str[startpos] : _str, t) : 0;
-  int pos = p? ((p - _str) + (strlen(t) - 1)) : (_len - 1);
+  size_t pos = p? ((p - _str) + (strlen(t) - 1)) : (_len - 1);
    
   return after(pos);
 }
 
-SubString String::after(char c, int startpos /* = 0 */)
+GL_SubString GL_String::after(char c, size_t startpos /* = 0 */)
 {
   assert (startpos <= _len);
   
   char* p = _len? strchr((startpos > 0)? &_str[startpos] : _str, c) : 0;
 
-  return p? after(p - _str) : after(_len - 1);
+  assert ((p - _str) >= 0);
+  return p? after((size_t)(p - _str)) : after(_len - 1);
+}
+
+GL_SubString GL_String::afterlast(char c, size_t startpos /* = 0 */)
+{
+  assert (startpos <= _len);
+  
+  char* p = _len? strrchr((startpos > 0)? &_str[startpos] : _str, c) : 0;
+
+  assert ((p - _str) >= 0);
+  return p? after((size_t)(p - _str)) : after(_len - 1);
 }
 
 // Helper routines for allocation, copying, etc.
 
-char* String::salloc(int len, int keep, bool force)
+char* GL_String::salloc(size_t len, size_t keep, bool force)
 {
   assert (len >= 0);
   assert ((keep >= 0) && (keep <= _len));
@@ -216,7 +247,7 @@ char* String::salloc(int len, int keep, bool force)
   return old_str;
 }
   
-void String::scopy(const char* t, int startpos, int len)
+void GL_String::scopy(const char* t, size_t startpos, size_t len)
 {
   assert (t);
   assert ((startpos >= 0) && (startpos <= _len));
@@ -228,7 +259,7 @@ void String::scopy(const char* t, int startpos, int len)
 }
 
 
-void String::sinsert(const char* t, int pos, int len, int repl)
+void GL_String::sinsert(const char* t, size_t pos, size_t len, size_t repl)
 {
   assert ((pos + repl >= 0) && (pos + repl <= _len));
   // if t and _str overlap we have to make a copy!
@@ -238,7 +269,7 @@ void String::sinsert(const char* t, int pos, int len, int repl)
   if ((old_str != _str) && (old_str != _nullString)) GL_free(old_str);
 }
 
-void String::sinsert(char c, int pos, int repl)
+void GL_String::sinsert(char c, size_t pos, size_t repl)
 {
   assert ((pos + repl >= 0) && (pos + repl <= _len));
   char* old_str = salloc(_len + 1 - repl, pos);
@@ -248,24 +279,24 @@ void String::sinsert(char c, int pos, int repl)
 }
 
 
-// ------------------- SubString methods ---------------------
+// ------------------- GL_SubString methods ---------------------
 
 
-SubString& SubString::operator =  (const String&     y) 
+GL_SubString& GL_SubString::operator =  (const GL_String&     y) 
 {
   _str.sinsert(y.chars(), _pos, y.length(), _len); 
   _len = y.length();
   return *this;
 }
 
-SubString& SubString::operator =  (const SubString&  y) 
+GL_SubString& GL_SubString::operator =  (const GL_SubString&  y) 
 {
   _str.sinsert(y.chars(), _pos, y.length(), _len); 
   _len = y.length();
   return *this;
 }
 
-SubString& SubString::operator =  (const char* t) 
+GL_SubString& GL_SubString::operator =  (const char* t) 
 {
   if (t) {
      int t_len = strlen(t);
@@ -277,7 +308,7 @@ SubString& SubString::operator =  (const char* t)
   return *this;
 }
 
-SubString& SubString::operator =  (char        c) 
+GL_SubString& GL_SubString::operator =  (char        c) 
 {
   _str.sinsert(c, _pos, _len); 
   _len = 1;
@@ -289,12 +320,12 @@ SubString& SubString::operator =  (char        c)
 // ------------------- Global functions ---------------------
 
 
-int compare(const String& x, const String& y) 
+int compare(const GL_String& x, const GL_String& y) 
 {
   return strncmp(x.chars(), y.chars(), MAX(x.length(), y.length()));
 }
 
-int compare(const String& x, const char* t) 
+int compare(const GL_String& x, const char* t) 
 {
   int res = strncmp(x.chars(), t, x.length());
   if (!res) {
@@ -303,7 +334,7 @@ int compare(const String& x, const char* t)
   return res;
 }
 
-int compare(const SubString& x, const String& y) 
+int compare(const GL_SubString& x, const GL_String& y) 
 {
   int res = strncmp(x.chars(), y.chars(), x.length());
   if (!res) {
@@ -312,7 +343,7 @@ int compare(const SubString& x, const String& y)
   return res;
 }
 
-int compare(const String& x, const SubString&  y) 
+int compare(const GL_String& x, const GL_SubString&  y) 
 {
   int res = strncmp(y.chars(), x.chars(), y.length());
   if (!res) {
@@ -321,7 +352,7 @@ int compare(const String& x, const SubString&  y)
   return -res;
 }
 
-int compare(const SubString& x, const SubString&  y) 
+int compare(const GL_SubString& x, const GL_SubString&  y) 
 {
   int res;
   int diff = x.length() - y.length();
@@ -337,7 +368,7 @@ int compare(const SubString& x, const SubString&  y)
   return res;
 }
 
-int compare(const SubString& x, const char* t) 
+int compare(const GL_SubString& x, const char* t) 
 {
   int res = strncmp(x.chars(), t, x.length());
   if (!res) {
@@ -354,7 +385,7 @@ int compare(const SubString& x, const char* t)
 
 #ifdef DEBUG	/* module self test */
 
-void String_Error(String S, const char* msg)
+void GL_StringError(GL_String S, const char* msg)
 {
    fprintf(stderr, "String error: %s is %s\n", msg, (const char*)S );
    exit(1);
@@ -362,13 +393,13 @@ void String_Error(String S, const char* msg)
 
 int main(int argc, char **argv)
 {
-   String x = "Hello";
-   String y = "world";
-   String n = "123";
-   String z(0, 31);
-   char*  s = ",";
-   String lft, mid, rgt;
-   String words[10];
+   GL_String x = "Hello";
+   GL_String y = "world";
+   GL_String n = "123";
+   GL_String z(0, 31);
+   const char*  s = ",";
+   GL_String lft, mid, rgt;
+   GL_String words[10];
    words[0] = "a";
    words[1] = "b";
    words[2] = "c";
@@ -378,10 +409,10 @@ int main(int argc, char **argv)
      { puts("WARNING: Arguments will be ignored!\n");
      } /* fi */
    
-   printf("Testing String class...\n");
-   if( z.length() != 0 ) String_Error(z, "z.length() = 0");  
+   printf("Testing GL_String class...\n");
+   if( z.length() != 0 ) GL_StringError(z, "z.length() = 0");  
   
-   /* The following tests were taken from the specification of the String class in the
+   /* The following tests were taken from the specification of the GL_String class in the
     * GNU libg++ library. At this point only very few selected methods are really 
     * implemented (and hence tested). This will be extended as need arises...
     */
@@ -389,7 +420,7 @@ int main(int argc, char **argv)
    /* Comparing, Searching, and Matching */
    
    /*  x.index("lo") 
-           returns the zero-based index of the leftmost occurrence of substring "lo" (3, in this case). The argument may be a String, SubString, char, char*, or
+           returns the zero-based index of the leftmost occurrence of substring "lo" (3, in this case). The argument may be a GL_String, GL_SubString, char, char*, or
            Regex. */   
    /*  x.index("l", 2) 
              returns the index of the first of the leftmost occurrence of "l" found starting the search at position x[2], or 2 in this case. */
@@ -400,7 +431,7 @@ int main(int argc, char **argv)
    /*  pos = r.search("leo", 3, len, 0) 
              returns the index of r in the char* string of length 3, starting at position 0, also placing the length of the match in reference parameter len.  */
    /*  x.contains("He") 
-             returns nonzero if the String x contains the substring "He". The argument may be a String, SubString, char, char*, or Regex.  */
+             returns nonzero if the string x contains the substring "He". The argument may be a GL_String, GL_SubString, char, char*, or Regex.  */
    /*  x.contains("el", 1) 
              returns nonzero if x contains the substring "el" at position 1. As in this example, the second argument to contains, if present, means to match the
            substring only at that position, and not to search elsewhere in the string.  */
@@ -409,23 +440,23 @@ int main(int argc, char **argv)
    /*  x.matches("lo", 3) 
              returns nonzero if x starting at position 3 exactly matches "lo", with no trailing characters (as it does in this example). */
    /*  x.matches(r) 
-             returns nonzero if String x as a whole matches Regex r.  */
-   int f = x.freq("l");		if( f != 2 ) String_Error( x, "x.freq(\"l\")" );
+             returns nonzero if string x as a whole matches Regex r.  */
+   int f = x.freq("l");		if( f != 2 ) GL_StringError( x, "x.freq(\"l\")" );
           /* returns the number of distinct, nonoverlapping matches to the argument (2 in this case). */
    
    /* Substring extraction */
    
-   z = x.at(2, 3);		if( z != "llo" ) String_Error( z, "x.at(2,3)" );
-	 /* sets String z to be equal to the length 3 substring of String x
+   z = x.at(2, 3);		if( z != "llo" ) GL_StringError( z, "x.at(2,3)" );
+	 /* sets string z to be equal to the length 3 substring of string x
 	    starting at zero-based position 2, setting z to "llo" in this case. A
-	    nil String is returned if the arguments don't make sense. */
+	    nil string is returned if the arguments don't make sense. */
 
-   x.at(2, 2) = "r";		if( x != "Hero" ) String_Error( x, "x.at(2,2) = \"r\"" );
+   x.at(2, 2) = "r";		if( x != "Hero" ) GL_StringError( x, "x.at(2,2) = \"r\"" );
 	 /* Sets what was in positions 2 to 3 of x to "r", setting x to "Hero" in
-	    this case. As indicated here, SubString assignments may be of different
+	    this case. As indicated here, GL_SubString assignments may be of different
 	    lengths. */
             x = "Hello";
-   x.at("He") = "je";		if( x != "jello" ) String_Error( x, "x.at(\"He\") = \"je\"" );
+   x.at("He") = "je";		if( x != "jello" ) GL_StringError( x, "x.at(\"He\") = \"je\"" );
 	 /*  x("He") is the substring of x that matches the first occurrence of it's
 	     argument. The substitution sets x to "jello". If "He" did not occur,
 	     the substring would be nil, and the assignment would have no effect. */
@@ -435,68 +466,68 @@ int main(int argc, char **argv)
 	     "Helio". */
    /* z = x.at(r); */
 	  /* sets String z to the first match in x of Regex r, or "ello" in this
-	     case. A nil String is returned if there is no match. */
-   z = x.before("o");		if( z != "Hell" ) String_Error( z, "x.before(\"o\")"  );
+	     case. A nil string is returned if there is no match. */
+   z = x.before("o");		if( z != "Hell" ) GL_StringError( z, "x.before(\"o\")"  );
 	  /* sets z to the part of x to the left of the first occurrence of "o", or
-	     "Hell" in this case. The argument may also be a String, SubString, or
+	     "Hell" in this case. The argument may also be a GL_String, GL_SubString, or
 	     Regex. (If there is no match, z is set to "".) */
-   x.before("ll") = "Bri";	if( x != "Brillo" ) String_Error( x, "x.before(\"ll\") = \"Bri\"" );
+   x.before("ll") = "Bri";	if( x != "Brillo" ) GL_StringError( x, "x.before(\"ll\") = \"Bri\"" );
 	  /* sets the part of x to the left of "ll" to "Bri", setting x to "Brillo". */
              x = "Hello";
-   z = x.before(2);		if( z != "He" ) String_Error( z, "x.before(2)" );
+   z = x.before(2);		if( z != "He" ) GL_StringError( z, "x.before(2)" );
 	  /* sets z to the part of x to the left of x[2], or "He" in this case. */
-   z = x.after("Hel");		if( z != "lo" ) String_Error( z, "x.after(\"Hel\")" );
+   z = x.after("Hel");		if( z != "lo" ) GL_StringError( z, "x.after(\"Hel\")" );
 	  /* sets z to the part of x to the right of "Hel", or "lo" in this case. */
    /* z = x.through("el")
 	     sets z to the part of x up and including "el", or "Hel" in this case. */
    /* z = x.from("el")
 	     sets z to the part of x from "el" to the end, or "ello" in this case. */
-   x.after("Hel") = "p";	if( x != "Help" ) String_Error( x, "x.after(\"Hel\") = \"p\"" );
+   x.after("Hel") = "p";	if( x != "Help" ) GL_StringError( x, "x.after(\"Hel\") = \"p\"" );
 	  /* sets x to "Help"; */
              x = "Hello";
-   z = x.after(3);		if( z != "o" ) String_Error( z, "x.after(3)" );
+   z = x.after(3);		if( z != "o" ) GL_StringError( z, "x.after(3)" );
           /* sets z to the part of x to the right of x[3] or "o" in this case. */
    /* z = " ab c"; z = z.after(RXwhite)
 	     sets z to the part of its old string to the right of the first group of
 	     whitespace, setting z to "ab c"; Use gsub(below) to strip out multiple
 	     occurrences of whitespace or any pattern. */
-   x[0] = 'J';			if( x != "Jello" ) String_Error( x, "x[0] = J" );
+   x[0] = 'J';			if( x != "Jello" ) GL_StringError( x, "x[0] = J" );
 	  /* sets the first element of x to 'J'. x[i] returns a reference to the ith
 	     element of x, or triggers an error if i is out of range. */
              x = "Hello";									  
    /* common_prefix(x, "Help")
-	     returns the String containing the common prefix of the two Strings or
+	     returns the string containing the common prefix of the two strings or
 	     "Hel" in this case. */
    /* common_suffix(x, "to")
-	     returns the String containing the common suffix of the two Strings or
+	     returns the string containing the common suffix of the two strings or
 	     "o" in this case. */
    
    
    /* Concatenation */
 
-   z = x + s + ' ' + y.at("w") + y.after("w") + "."; if( z != "Hello, world." ) String_Error( z, "z = ..." );
+   z = x + s + ' ' + y.at("w") + y.after("w") + "."; if( z != "Hello, world." ) GL_StringError( z, "z = ..." );
 	 /*  sets z to "Hello, world." */
-   x += y;		if( x != "Helloworld" ) String_Error( x, "x += y" );
+   x += y;		if( x != "Helloworld" ) GL_StringError( x, "x += y" );
 	 /*  sets x to "Helloworld" */
          x = "Hello";
    /* cat(x, y, z)
 	     A faster way to say z = x + y. */
    /* cat(z, y, x, x)
 	     Double concatenation; A faster way to say x = z + y + x. */
-   y.prepend(x);	if( y != "Helloworld" ) String_Error( y, "y.prepend(x)" );
+   y.prepend(x);	if( y != "Helloworld" ) GL_StringError( y, "y.prepend(x)" );
 	 /*  A faster way to say y = x + y. */
              y = "world";
    /* z = replicate(x, 3);
 	     sets z to "HelloHelloHello". */
    /* z = join(words, 3, "/")
-	     sets z to the concatenation of the first 3 Strings in String array
+	     sets z to the concatenation of the first 3 strings in string array
 	     words, each separated by "/", setting z to "a/b/c" in this case. The
 	     last argument may be "" or 0, indicating no separation. */
 
    /* Other manipulations */
 
    /* z = "this string has five words"; i = split(z, words, 10, RXwhite);
-	     sets up to 10 elements of String array words to the parts of z
+	     sets up to 10 elements of string array words to the parts of z
 	     separated by whitespace, and returns the number of parts actually
 	     encountered (5 in this case). Here, words[0] = "this", words[1] =
 	     "string", etc. The last argument may be any of the usual. If there is
@@ -507,7 +538,7 @@ int main(int argc, char **argv)
 	     "Hellllo". The first argument may be any of the usual, including Regex.
 	     If the second argument is "" or 0, all occurrences are deleted. gsub
 	     returns the number of matches that were replaced. */
-   z = x + y; z.del("loworl");		if( z != "Held" ) String_Error( z, "z.del()" );
+   z = x + y; z.del("loworl");		if( z != "Held" ) GL_StringError( z, "z.del()" );
 	 /*  deletes the leftmost occurrence of "loworl" in z, setting z to "Held". */
    /* z = reverse(x)
 	     sets z to the reverse of x, or "olleH". */
@@ -524,21 +555,21 @@ int main(int argc, char **argv)
    
    /* Some regression test */
 
-   String s1("ab");
-   String s2("abc");
-   if( s1 == s2 ) String_Error("TRUE", "'ab' == 'abc'");
+   GL_String s1("ab");
+   GL_String s2("abc");
+   if( s1 == s2 ) GL_StringError("TRUE", "'ab' == 'abc'");
   
-   String pes("proc1 proc2");
+   GL_String pes("proc1 proc2");
    pes += ' ';   
    pes = pes.after(' ');
-   if( pes != "proc2 " ) String_Error(pes, "pes = pes.after()");      
+   if( pes != "proc2 " ) GL_StringError(pes, "pes = pes.after()");      
 
    pes.form("%c1%s3%d5%u7%x9%o%%", '%', "2", 4, 6, 8, 0);
-   if( pes != "%1234567890%" ) String_Error(pes, "pes.form() = '%1234567890%'");
+   if( pes != "%1234567890%" ) GL_StringError(pes, "pes.form() = '%1234567890%'");
 
-   String tst("Test\0Test", 9);
-   if( tst.length() != 9 ) String_Error(tst, "tst.length() = 9");
-   if( tst[4] != '\0' || tst[5] != 'T' ) String_Error(tst, "tst = 'Test\\0Test'");
+   GL_String tst("Test\0Test", 9);
+   if( tst.length() != 9 ) GL_StringError(tst, "tst.length() = 9");
+   if( tst[4] != '\0' || tst[5] != 'T' ) GL_StringError(tst, "tst = 'Test\\0Test'");
      
    exit(0);
 }
