@@ -4,10 +4,12 @@
 /* Author: Rainer Doemer						*/
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 12/10/14 */
 
 /* modifications: (most recent first)
  *
+ * 12/10/14 SD	Adjustments for modern Linux support
+ * 01/07/06 RD	avoid warning about non-virtual destructor
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 05/16/04 PC	buffered and piped keyword was inlcuded under '_specc' scope
  * 05/14/04 PC	signal keyword was inlcuded under '_specc' scope
@@ -153,14 +155,23 @@ class Empty
 class I
 {
 public:
+	I(void);
+	virtual ~I(void);
 	virtual int Method1(void ) = 0;
 	virtual int Method2(void ) = 0;
 };
+I::I(void) {}
+I::~I(void) {}
+
 class I2
 {
 public:
+	I2(void);
+	virtual ~I2(void);
 	virtual int Method3(void ) = 0;
 };
+I2::I2(void) {}
+I2::~I2(void) {}
 
 class C0 : public _specc::channel
 {
@@ -249,14 +260,14 @@ struct BufferStruct
 {
 char x1;
 _specc::buffered<char> x2;
-BufferStruct::BufferStruct() : x2(event(&clk)) { }
+BufferStruct() : x2(event(&clk)) { }
 };
 
 struct BufferStruct2
 {
 _specc::buffered<char> x2;
 char x1;
-BufferStruct2::BufferStruct2() : x2(event(&clk)) { }
+BufferStruct2() : x2(event(&clk)) { }
 };
 
 int		x;
