@@ -4,10 +4,12 @@
 /* Author: Rainer Doemer			first version: 02/10/98 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 01/09/03 RD	added support for implicit clock (auto-event)
  * 12/17/02 RD	added support for asynchronous reset signal
@@ -62,7 +64,7 @@ SIR_Declarator::DeclTypes	= DeclTypes;
 SIR_Declarator::Parameters	= NULL;
 SIR_Declarator::ParamScope	= NULL;
 if (SymbolName)
-   { SIR_Declarator::SymbolName	= new string(SymbolName);
+   { SIR_Declarator::SymbolName	= new gl_string(SymbolName);
     } /* fi */
 else
    { SIR_Declarator::SymbolName	= NULL;
