@@ -4,10 +4,11 @@
 /* Author: Rainer Doemer			first version: 10/10/97 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 11/08/01 RD	removed SIR_ERROR_WRITING_SPECC_FILE
  * 05/31/01 RD	removed code not needed for the SCRC
@@ -41,7 +42,10 @@
 /*** exported variables *************************************************/
 
 
-string		SIR_ErrMsg;
+//++++++++++++++++++++++++++++ API Layer 1 +++++++++++++++++++++++++++++//
+
+
+gl_string	SIR_ErrMsg;
 ERROR		SIR_Error = SIR_ERROR_NO_ERROR;	/* last error of this module */
 
 
