@@ -4,11 +4,17 @@
 /* Author: Rainer Doemer			first version: 02/10/98 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
  * 06/15/04 PC  Adjustments for scrc 2.0
+ * 12/19/02 RD	added SIR_Initializer::IsComposite()
+ * 03/29/02 RD	added SIR_Initializer::GetAutoArraySize()
+ * 01/17/02 RD	added SIR_Initializer::GetIndex()
+ * 01/16/02 RD	bug fix in recursion of SIR_Initializer::Print()
  * 01/15/02 RD	added support for composite annotations
  * 11/21/01 RD	took out default arguments from function definitions
  * 11/13/01 RD	added line wrapping in code generation
@@ -124,40 +130,40 @@ return(SIR_ERROR_NO_ERROR);
 
 } /* end of SIR_Initializer::DFS_ForAllNodes */
 
-BOOL SIR_Initializer::IsComposite(      /* checks if initializer is composite */
-        BOOL            TargetIsArray)
+bool SIR_Initializer::IsComposite(      /* checks if initializer is composite */
+        bool            TargetIsArray)
 {
 
 if (! this)
-   { return(FALSE);
+   { return(false);
     } /* fi */
 
 if (InitList)
-   { return(TRUE);
+   { return(true);
     } /* fi */
 
 assert(Initializer != NULL);
 
 if (  (TargetIsArray)
     &&(Initializer->Type == SIR_CONST_CHARSTRING))
-   { return(TRUE);
+   { return(true);
     } /* fi */
 
-return(FALSE);
+return(false);
 
 } /* end of SIR_Initializer::IsComposite */
 
 
 ERROR SIR_Initializer::WriteSC(	/* (re-) generates SpecC source code */
-	GL_HANDLE	*IO,
-	BOOL		CplusplusMode,
-	BOOL		StartNewLine /* = TRUE */)
+	gl_io		*IO,
+	bool		CplusplusMode,
+	bool		StartNewLine /* = true */)
 {
 sir_initializer	*Init;
 
 if (InitList)
    { if (StartNewLine)
-	{ SIR_LineInfo::WriteVSPACE(IO, TRUE);
+	{ SIR_LineInfo::WriteVSPACE(IO, true);
 	 } /* fi */
      IO->PutS("{ ");
      IO->Add2Tab(2);
@@ -193,12 +199,12 @@ return(SIR_ERROR_NO_ERROR);
 
 } /* end of SIR_Initializer::WriteSC */
 
-const char *SIR_Initializer::Print(     /* create SpecC source code (1 line) */
-        string          *Buffer /* = NULL */)   /* (NULL = internal buffer) */
+const char *SIR_Initializer::Print(	/* create SpecC source code (1 line) */
+	gl_string	*Buffer /* = NULL */)	/* (NULL = internal buffer) */
 {
-static string   PrintBuffer;
-string          *UseBuffer;
-sir_initializer *Init;
+static gl_string PrintBuffer;
+gl_string	*UseBuffer;
+sir_initializer	*Init;
 
 if (Buffer)
    { UseBuffer = Buffer;
