@@ -4,10 +4,15 @@
 /* Author: Rainer Doemer			first version: 09/24/97 */
 /************************************************************************/
 
-/* last update: 11/16/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 01/15/02 RD	added support for composite annotations
+ * 11/16/01 RD	extended convenience functions for code generation
  * 11/08/01 RD	switched code generation to use GL_IO layer
  * 05/31/01 RD	eliminated level 2 of SIR API
  * 05/25/01 RD	eliminated support for binary SIR files (import/export)
@@ -46,7 +51,7 @@ class SIR_Design :		/* root of the data structure */
 	public SIR_Unit			/* is a unit of the design */
 {
 public:
-string		Name;		/* name of this design */
+gl_string	Name;		/* name of this design */
 sir_file_list	*FileList;	/* list of (source) files */
 sir_import_list	*ImportList;	/* list of imported (binary) files */
 sir_types	*Types;		/* list of all types in the description */
@@ -100,23 +105,23 @@ void GarbageCollection(void);	/* garbage collector */
 
 
 ERROR WriteSC(			/* (re-) generates SpecC source code */
-	GL_HANDLE	*IO,
+	gl_io		*IO,
 	const char	*Filename,	/* (full path) */
-	BOOL		WriteNotes = TRUE,
-	BOOL		WriteLines = TRUE);
+	bool		WriteNotes = true,
+	bool		WriteLines = true);
 
 ERROR WriteH(			/* generates C++ source code: header file */
-	GL_HANDLE	*IO,
+	gl_io		*IO,
 	const char	*Filename,	/* (full path) */
-	BOOL		WriteNotes = TRUE,
-	BOOL		WriteLines = TRUE);
+	bool		WriteNotes = true,
+	bool		WriteLines = true);
 
 ERROR WriteCC(			/* generates C++ source code: main file */
-	GL_HANDLE	*IO,
+	gl_io		*IO,
 	const char	*Filename,	/* (full path) */
 	const char	*Headername,	/* (full path) */
-	BOOL		WriteNotes = TRUE,
-	BOOL		WriteLines = TRUE);
+	bool		WriteNotes = true,
+	bool		WriteLines = true);
 };
 
 
