@@ -4,10 +4,11 @@
 /* Author: Rainer Doemer			first version: 10/08/97 */
 /************************************************************************/
 
-/* last update: 09/26/06 */
+/* last update: 12/15/14 */
 
 /* modifications: (most recent first)
  *
+ * 12/15/14 RD	finalized scrc_V22
  * 09/26/06 PC  Adjustments for scrc 2.1
  * 06/03/05 RD	reorganized and renamed global type names
  * 04/13/05 RD	bug fix: WriteIntro() argument was pointer instead of bool
@@ -627,7 +628,8 @@ if (AtTopLevel)
 #endif /* SIR_PUT_COMMENTS_IN_SOURCE */
 
      SIR_Symbols::WriteIntro(IO, false,
-			&PutIntro, &PutSeparator, &PutComment,
+			&PutIntro, &PutSeparator,
+			PutComment,	/* bug fix 04/13/05, RD */
 			"global annotations");
     } /* fi */
 
