@@ -4,10 +4,13 @@
 /* Author: Rainer Doemer			first version: 10/08/97 */
 /************************************************************************/
 
-/* last update: 06/15/04 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 01/14/05 RD	added inclusion of IntRep/Extern.h
  * 06/15/04 PC  Adjustments for scrc 2.0
  * 02/13/03 RD	added support for notes at fsmd states
  * 02/12/03 RD	bug fix: FindLocallyViaTypeDef() only finds unnamed usertypes
@@ -82,7 +85,7 @@ assert(  (Class == SIR_USERTYPE_STRUCT)
 
 SIR_UserType::Class		= Class;
 if (Name)
-   { SIR_UserType::Name		= new string(Name);
+   { SIR_UserType::Name		= new gl_string(Name);
     } /* fi */
 else
    { SIR_UserType::Name		= NULL;
@@ -111,7 +114,7 @@ SIR_UserType::SIR_UserType(		/* constructor #3 (duplicator) */
 
 SIR_UserType::Class		= Original->Class;
 SIR_UserType::Name		= Original->Name ?
-					new String(*Original->Name) :
+					new GL_String(*Original->Name) :
 					NULL;
 SIR_UserType::Type		= Original->Type;
 SIR_UserType::Members		= Original->Members ?
@@ -141,7 +144,7 @@ delete Notes;
 } /* end of SIR_UserType::~SIR_UserType */
 
 
-BOOL SIR_UserType::IsNamed(void)	/* checks if Name is defined */
+bool SIR_UserType::IsNamed(void)	/* checks if Name is defined */
 {
 
 return(Name != NULL);
@@ -149,7 +152,7 @@ return(Name != NULL);
 } /* end of SIR_UserType::IsNamed */
 
 
-BOOL SIR_UserType::IsDefinition(void)	/* checks if Members are defined */
+bool SIR_UserType::IsDefinition(void)	/* checks if Members are defined */
 {
 
 return(Members != NULL);
@@ -157,7 +160,7 @@ return(Members != NULL);
 } /* end of SIR_UserType::IsDefinition */
 
 
-BOOL SIR_UserType::IsSameStructure(	/* checks if structure is the same */
+bool SIR_UserType::IsSameStructure(	/* checks if structure is the same */
 	sir_usertype	*Other)
 {
 sir_member	*Member1,
@@ -166,35 +169,35 @@ sir_member	*Member1,
 assert(Other != NULL);
 
 if (this->Color == SIR_RED)	/* loop in recursion? */
-   { return(TRUE);	/* we are checking this already */
+   { return(true);	/* we are checking this already */
     } /* fi */
 
 if (this == Other)	/* identity? */
-   { return(TRUE);
+   { return(true);
     } /* fi */
 
 if (  ((Name) && (!Other->Name))
     ||((!Name) && (Other->Name)))
-   { return(FALSE);
+   { return(false);
     } /* fi */
 if (  (Name)
     &&(0 != strcmp(Name->chars(), Other->Name->chars())))
-   { return(FALSE);
+   { return(false);
     } /* fi */
 if (  ((Members) && (!Other->Members))
     ||((!Members) && (Other->Members)))
-   { return(FALSE);
+   { return(false);
     } /* fi */
 if (! Members)
    { assert(!Other->Members);
-     return(TRUE);
+     return(true);
     } /* fi */
 
 Member1 = Members->First();
 Member2 = Other->Members->First();
 while(Member1)
    { if (!Member2)
-	{ return(FALSE);
+	{ return(false);
 	 } /* fi */
 
      assert(this->Color == SIR_WHITE);
@@ -203,7 +206,7 @@ while(Member1)
 						false, true))	/* recursion */
 	{ assert(this->Color == SIR_RED);
 	  this->Color = SIR_WHITE;	/* remove the mark */
-	  return(FALSE);
+	  return(false);
 	 } /* fi */
      assert(this->Color == SIR_RED);
      this->Color = SIR_WHITE;	/* remove the mark */
@@ -212,30 +215,30 @@ while(Member1)
 	{ if (  (!Member2->Symbol)
 	      ||(0 != strcmp(Member1->Symbol->Name.chars(),
 				Member2->Symbol->Name.chars())))
-	     { return(FALSE);
+	     { return(false);
 	      } /* fi */
 	 } /* fi */
      else
 	{ if (Member2->Symbol)
-	     { return(FALSE);
+	     { return(false);
 	      } /* fi */
 	 } /* esle */
      if (Member1->BitFieldSize != Member2->BitFieldSize)
-	{ return(FALSE);
+	{ return(false);
 	 } /* fi */
      Member1 = Member1->Succ();
      Member2 = Member2->Succ();
     } /* elihw */
 if (Member2)
-   { return(FALSE);
+   { return(false);
     } /* fi */
 
-return(TRUE);
+return(true);
 
 } /* end of SIR_UserType::IsSameStructure */
 
 
-BOOL SIR_UserType::ContainsNested(	/* checks for nested struct/union */
+bool SIR_UserType::ContainsNested(	/* checks for nested struct/union */
 	sir_usertype	*UserType)	/* (struct/union definitions only) */
 {
 sir_member	*Member;
@@ -253,28 +256,28 @@ while(Member)
 	 } /* elihw */
      if (MemberType->UserType)
 	{ if (MemberType->UserType == UserType)
-	     { return(TRUE);		/* this contains def. of UserType */
+	     { return(true);		/* this contains def. of UserType */
 	      } /* fi */
 	  if (  (MemberType->UserType->Class != SIR_USERTYPE_ENUM)
 	      &&(MemberType->UserType->ContainsNested(UserType)))
-	     { return(TRUE);		/* child contains def. of UserType */
+	     { return(true);		/* child contains def. of UserType */
 	      } /* fi */
 	 } /* fi */
      if (! UserType->Name)
 	{ /* unnamed, typedef'd user-type (Bug fix 07/01/00, R.D.) */
 	  if (Member->Type->SubTypeTreeContains(UserType->Type))
-	     { return(TRUE);		/* dependency by use of typedef */
+	     { return(true);		/* dependency by use of typedef */
 	      } /* fi */
 	 } /* fi */
      Member = Member->Succ();
     } /* elihw */
 
-return(FALSE);	/* not found */
+return(false);	/* not found */
 
 } /* end of SIR_UserType::ContainsNested */
 
 
-BOOL SIR_UserType::LocalUTypeNeedsDef(	/* check whether a user-type member */
+bool SIR_UserType::LocalUTypeNeedsDef(	/* check whether a user-type member */
 	sir_member	*Member)	/* needs a local definition	    */
 {
 sir_member	*PrevMember;
@@ -289,18 +292,18 @@ if (Member->Type->UserType)	/* is a user-type member? */
 	{ PrevMember = Member->Pred();		/* is not defined before? */
 	  while(PrevMember)
 	     { if (Member->Type == PrevMember->Type)
-		  { return(FALSE);	/* already locally defined before */
+		  { return(false);	/* already locally defined before */
 		   } /* fi */
 	       PrevMember = PrevMember->Pred();
 	      } /* elihw */
-	  return(TRUE);		/* yes, we do need a local definition here */
+	  return(true);		/* yes, we do need a local definition here */
 	 } /* fi */
      else
-	{ return(FALSE);	/* somewhere else defined */
+	{ return(false);	/* somewhere else defined */
 	 } /* esle */
     } /* fi */
 else
-   { return(FALSE);	/* not a user-type */
+   { return(false);	/* not a user-type */
     } /* esle */
 
 } /* end of SIR_UserType::LocalUTypeNeedsDef */
@@ -562,7 +565,7 @@ switch(Class)
 	{ return("enum");
 	 }
      default:
-	{ assert(FALSE);	/* bad usertype class */
+	{ assert(false);	/* bad usertype class */
 	 }
     } /* hctiws */
 
@@ -585,10 +588,10 @@ else
 
 
 const char *SIR_UserType::PrettyString(		/* creates a SpecC string */
-	string		*Buffer,		/* (one of the ones below) */
-	BOOL		IncludeNotes,
-	BOOL		CplusplusMode /* = FALSE */,
-	BOOL		UserTypeDefReq /* = FALSE */)
+	gl_string	*Buffer,		/* (one of the ones below) */
+	bool		IncludeNotes,
+	bool		CplusplusMode /* = false */,
+	bool		UserTypeDefReq /* = false */)
 {
 
 assert(Buffer != NULL);
@@ -614,7 +617,7 @@ return(PrettyStringD(Buffer, IncludeNotes, CplusplusMode));
 
 
 const char *SIR_UserType::PrettyStringN(	/* PrettyString, e.g.	*/
-	string		*Buffer)		/* "struct Name"	*/
+	gl_string	*Buffer)		/* "struct Name"	*/
 {
 
 assert(Buffer != NULL);
@@ -630,7 +633,7 @@ return(Buffer->chars());
 
 
 const char *SIR_UserType::PrettyStringT(	/* PrettyString, e.g.	*/
-	string		*Buffer)		/* "TypeDefName"	*/
+	gl_string	*Buffer)		/* "TypeDefName"	*/
 {
 
 assert(Buffer != NULL);
@@ -644,11 +647,11 @@ return(Buffer->chars());
 
 
 const char *SIR_UserType::PrettyStringD(	/* PrettyString, e.g.	*/
-	string		*Buffer,		/* "struct [Name] { ... }" */
-	BOOL		IncludeNotes,
-	BOOL		CplusplusMode)
+	gl_string	*Buffer,		/* "struct [Name] { ... }" */
+	bool		IncludeNotes,
+	bool		CplusplusMode)
 {
-string		LocalBuffer,
+gl_string	LocalBuffer,
 		TmpString;
 sir_member	*Member;
 
@@ -718,12 +721,12 @@ return(Buffer->chars());
 
 
 ERROR SIR_UserType::WriteSC(	/* (re-) generates SpecC source code */
-	GL_HANDLE	*IO,	/* (for usertype _definitions_) */
-	BOOL		WriteNotes,
-	BOOL		CplusplusMode)
+	gl_io		*IO,	/* (for usertype _definitions_) */
+	bool		WriteNotes,
+	bool		CplusplusMode)
 {
 sir_member	*Member;
-string		Buffer;
+gl_string	Buffer;
 
 assert(IsDefinition());	/* don't handle declarations here */
 
@@ -778,7 +781,7 @@ else
 	      } /* fi */
 	  else
 	     { /* unnamed structs/unions will be defined implicitly! */
-	       assert(FALSE);	/* this case is not handled here */
+	       assert(false);	/* this case is not handled here */
 	      } /* esle */
 	 } /* esle */
     } /* esle */
@@ -1258,7 +1261,7 @@ switch(Class)
 	 }
      default:
 	{ TypeType = SIR_TYPE_ANY_TYPE; /* just to keep the compiler quiet */
-	  assert(FALSE);	/* bad usertype */
+	  assert(false);	/* bad usertype */
 	 }
     } /* hctiws */
 
@@ -1358,7 +1361,7 @@ while(Curr())
 	 } /* fi */
      if (0 == (CmpVal = strcmp(NewUserType->Name->chars(),
 				Curr()->Name->chars())))
-	{ assert(FALSE);	/* already exists! */
+	{ assert(false);	/* already exists! */
 	 } /* fi */
      if (CmpVal < 0)
 	{ break;
@@ -1503,7 +1506,7 @@ while(UserType)
 	  if (NumUsers > 1)		// used by more than one symbol?
 	     { Remove(UserType);		// take out of this table
 	       assert(UserType->Name == NULL);
-	       UserType->Name = new string(	// assign a new name
+	       UserType->Name = new gl_string(	// assign a new name
 				CreateNewName(SIR_CXX_ANONYMOUS_NAME));
 	       Insert(UserType);		// re-insert into this table
 	      } /* fi */
@@ -1517,7 +1520,7 @@ while(UserType)
 const char *SIR_UserTypes::CreateNewName(	/* generate a new name */
 	const char	*Format)		/* (must contain one %d) */
 {
-static string	NewName;
+static gl_string NewName;
 int		i;
 
 i = 0;
@@ -1534,15 +1537,15 @@ return(NewName.chars());
 const char *SIR_UserTypes::Print(	/* print SpecC code */
 	void)
 {
-GL_HANDLE	*IO;
-static string	PrintBuffer;
-BOOL		Dummy = FALSE;
+gl_io		*IO;
+static gl_string PrintBuffer;
+bool		Dummy = false;
 
 if (!(IO = GL_IO::OpenString(&PrintBuffer, GL_IO_WRITE)))
    { SIR_Error = GL_Error;
      return(NULL);
     } /* fi */
-if ((SIR_Error = WriteSC(IO, TRUE, FALSE, &Dummy)))
+if ((SIR_Error = WriteSC(IO, true, false, &Dummy)))
    { IO->Close();
      return(NULL);
     } /* fi */
@@ -1558,13 +1561,13 @@ return(PrintBuffer.chars());
 
 
 ERROR SIR_UserTypes::WriteSC(	/* (re-) generates SpecC (or C++) source code */
-	GL_HANDLE	*IO,
-	BOOL		WriteNotes,
-	BOOL		CplusplusMode,
-	BOOL		*PutSeparator)
+	gl_io		*IO,
+	bool		WriteNotes,
+	bool		CplusplusMode,
+	bool		*PutSeparator)
 {
 sir_usertype	*UserType;
-string		Buffer;
+gl_string	Buffer;
 bool		PutIntro,
 		PutIntroInit,
 		PutComments;
@@ -1713,12 +1716,12 @@ return(SIR_ERROR_NO_ERROR);
 
 ERROR SIR_UserTypes::WriteOrderedSC(	/* writes ordered structures/unions */
 	sir_usertype	*Current,	/* (recursively) */
-	GL_HANDLE	*IO,
-	BOOL		WriteNotes,
-	BOOL		CplusplusMode,
-	BOOL		*PutSeparator,
-	BOOL		*PutIntro,
-	BOOL		PutComments)
+	gl_io		*IO,
+	bool		WriteNotes,
+	bool		CplusplusMode,
+	bool		*PutSeparator,
+	bool		*PutIntro,
+	bool		PutComments)
 {
 sir_usertype	*UserType;
 
