@@ -4,10 +4,34 @@
 /* Author: Rainer Doemer			first version: 07/02/97 */
 /************************************************************************/
 
-/* last update: 11/21/01 */
+/* last update: 09/26/06 */
 
 /* modifications: (most recent first)
  *
+ * 09/26/06 PC  Adjustments for scrc 2.1
+ * 06/03/05 RD	reorganized and renamed global type names
+ * 06/15/04 PC  Adjustments for scrc 2.0
+ * 12/04/03 HY	added utility function PRS_CheckScope()
+ * 10/30/03 RD	added some control-flow checking (i.e. CheckControlFlow())
+ * 09/08/03 RD	added PRS_ERROR_INVALID_*_FOR_* errors
+ * 04/04/03 RD	added a couple of checks for parsing types from text strings
+ * 02/07/03 RD	added PRS_ERROR_MULTIPLE_LABELS_IN_FSMD
+ * 01/09/03 RD	added PRS_ERROR_CLOCK_PERIOD_NOT_POSITIVE
+ * 12/17/02 RD	added PRS_ERROR_SIGNAL_TYPE_MISMATCH_3,
+ *		PRS_ERROR_RESET_SPEC_OUTPUT_PORT
+ * 12/05/02 RD	added PRS_ERROR_SIGNAL_TYPE_MISMATCH_2
+ * 11/25/02 RD	renamed PRS_ERROR_BUFFER_EVENT_OUTPUT_PORT to
+ *		PRS_ERROR_CLOCK_SPEC_OUTPUT_PORT,
+ *		added PRS_ERROR_SENSITIVITY_OUTPUT_PORT,
+ *		PRS_ERROR_SIGNAL_TYPE_MISMATCH,
+ *		PRS_ERROR_NOTIFY_RISING_OR_FALLING_[12]
+ * 07/17/02 RD	added PRS_ERROR_BUFFER_EVENT_OUTPUT_PORT
+ * 07/16/02 RD	updated PRS_ERROR_EVENT_TYPE_MISMATCH
+ * 07/13/02 RD	strengthened type checking by PRS_CheckTypeDefinition()
+ * 07/10/02 RD	added PRS_ERROR_FSM_STATE_AMBIGUOUS,
+ *		added/replaced PRS_ERROR_INVALID_FSM_STATE_LABEL
+ * 05/06/02 RD	added PRS_ParseType(), SIR_Types::Obtain(Text)
+ * 01/16/02 RD	added support for parsing (composite) annotations
  * 11/21/01 RD	took out default arguments from function definitions
  * 11/15/01 RD	minor fixes to usage info and comments in PRS_ErrorText()
  * 11/08/01 RD	switched code generation to use GL_IO layer
@@ -62,7 +86,7 @@ unsigned int	PRS_NumberOfLines = 0;		/* number lines processed */
 /*** internal variables *************************************************/
 
 
-static string	ErrorMessage;
+static gl_string	ErrorMessage;
 
 
 /************************************************************************/
@@ -162,7 +186,7 @@ switch (ErrorNumber)
 	  return(ErrorMessage.chars()
 		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
 			"in file \"%s\":" GL_ERROR_MSG_NEWLINE
-		"Type mismatch: '%s' is not an event" */ );
+		"Type mismatch: '%s' is not an event or signal" */ );
      case PRS_ERROR_ILLEGAL_BITFIELD_SIZE:
 	  return(ErrorMessage.chars()
 		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
@@ -188,11 +212,12 @@ switch (ErrorNumber)
 		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
 			"in file \"%s\":" GL_ERROR_MSG_NEWLINE
 		"Empty filename" */ );
-     case PRS_ERROR_NOT_BEHAVIOR_INSTANCE:
+     case PRS_ERROR_INVALID_FSM_STATE_LABEL:
 	  return(ErrorMessage.chars()
 		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
 			"in file \"%s\":" GL_ERROR_MSG_NEWLINE
-		"Type mismatch: '%s' is not a behavior instance" */ );
+			"Invalid FSM state label;" GL_ERROR_MSG_NEWLINE
+			"'%s' is not behavior instance or local state" */ );
      case PRS_ERROR_STATE_LABEL_DEFINED_TWICE:
 	  return(ErrorMessage.chars()
 		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
@@ -274,6 +299,61 @@ switch (ErrorNumber)
      case PRS_ERROR_INTERRUPT_OUTPUT_PORT:
 	  return(ErrorMessage.chars()
 		/* "Interrupt event '%s' must not be output port" */ );
+     case PRS_ERROR_TYPE_PARSING_FAILED:
+	  return(ErrorMessage.chars()
+		/* "String conversion to SpecC type failed;"
+						GL_ERROR_MSG_NEWLINE
+		"parse error in '%s'" */ );
+     case PRS_ERROR_FSM_STATE_AMBIGUOUS:
+	  return(ErrorMessage.chars()
+		/* "FSM state name '%s' is ambiguous" */ );
+     case PRS_ERROR_CLOCK_SPEC_OUTPUT_PORT:
+	  return(ErrorMessage.chars()
+		/* "Clock specifier '%s' must not be output port" */ );
+     case PRS_ERROR_SENSITIVITY_OUTPUT_PORT:
+	  return(ErrorMessage.chars()
+		/* "Sensitivity event '%s' must not be output port" */ );
+     case PRS_ERROR_RESET_SPEC_OUTPUT_PORT:
+	  return(ErrorMessage.chars()
+		/* "Reset specifier '%s' must not be output port" */ );
+     case PRS_ERROR_SIGNAL_TYPE_MISMATCH:
+     case PRS_ERROR_SIGNAL_TYPE_MISMATCH_2:
+     case PRS_ERROR_SIGNAL_TYPE_MISMATCH_3:
+	  return(ErrorMessage.chars()
+		/* "Type mismatch: %s" */ );
+     case PRS_ERROR_NOTIFY_RISING_OR_FALLING_1:
+	  return("Event filters 'rising' and 'falling'" GL_ERROR_MSG_NEWLINE
+		"must not be used with 'notify' statement");
+     case PRS_ERROR_NOTIFY_RISING_OR_FALLING_2:
+	  return("Event filters 'rising' and 'falling'" GL_ERROR_MSG_NEWLINE
+		"must not be used with 'notifyone' statement");
+     case PRS_ERROR_CLOCK_PERIOD_NOT_POSITIVE:
+	  return(ErrorMessage.chars()
+		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
+			"in file \"%s\":" GL_ERROR_MSG_NEWLINE
+		"Positive value expected for clock period" */ );
+     case PRS_ERROR_MULTIPLE_LABELS_IN_FSMD:
+	  return(ErrorMessage.chars()
+		/* "Multiple labels ('%s' and '%s')" GL_ERROR_MSG_NEWLINE
+			"defined for one statement inside 'fsmd'" */ );
+     case PRS_ERROR_NOTES_RESTRICTED_TO_CLSBODY:
+	  return("Illegal annotation at class;" GL_ERROR_MSG_NEWLINE
+		"notes are restricted to class body");
+     case PRS_ERROR_NOTES_RESTRICTED_TO_CLSBODY2:
+	  return("Illegal global annotation;" GL_ERROR_MSG_NEWLINE
+		"notes are restricted to class body");
+     case PRS_ERROR_INVALID_EXPRESSION_FOR_AFTER:
+     case PRS_ERROR_INVALID_EXPRESSION_FOR_PIPED:
+	  return(ErrorMessage.chars()
+		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
+			"in file \"%s\":" GL_ERROR_MSG_NEWLINE
+		"Invalid expression for '%s' assignment" */ );
+     case PRS_ERROR_INVALID_COUNT_FOR_AFTER:
+     case PRS_ERROR_INVALID_COUNT_FOR_PIPED:
+	  return(ErrorMessage.chars()
+		/* "Parse error in line %u" GL_ERROR_MSG_NEWLINE
+			"in file \"%s\":" GL_ERROR_MSG_NEWLINE
+		"Negative clock cycle count in '%s' assignment" */ );
     } /* hctiws */
 
 return("Unknown error number (internal)");
@@ -385,6 +465,8 @@ ERROR PRS_CheckTypeDefinition(		/* checks if type is legal here */
 	sir_type	*TypeEntry,
 	bool		VoidIsOK /* = false */,
 	bool		IsPort /* = false */,
+	bool		IsParameter /* = false */,
+	bool		IsMember /* = false */,
 	SIR_STORAGE	StorageClass /* = SIR_STORAGE_NONE */,
 	unsigned int	Line /* = 0 */,
 	const char	*Filename /* = NULL */)
@@ -393,7 +475,7 @@ ERROR	SirError;
 
 assert(TypeEntry != NULL);
 
-if ((SirError = TypeEntry->Check(VoidIsOK, IsPort,
+if ((SirError = TypeEntry->Check(VoidIsOK, IsPort, IsParameter, IsMember,
 			(  (StorageClass == SIR_STORAGE_EXTERN)
 			 ||(StorageClass == SIR_STORAGE_TYPEDEF)))))
    { ErrorMessage.form(	"Illegal type error in line %u" GL_ERROR_MSG_NEWLINE
@@ -411,6 +493,26 @@ return(PRS_ERROR_NO_ERROR);
 } /* end of PRS_CheckTypeDefinition */
 
 
+sir_symbol* PRS_CheckScope(		/* checks the current scope */
+	sir_symbols	*CurrScope) /* (returns ClassSymbol if SCOPE_CLASS) */
+{
+
+if (CurrScope->ScopeInfo == SIR_SCOPE_GLOBAL)
+	return NULL; /* fi */
+else
+   {
+	if (CurrScope->ScopeInfo == SIR_SCOPE_CLASS)
+		return CurrScope->ParentSymbol; /* fi */
+	else
+	{
+		assert(CurrScope->Parent != NULL);
+		return PRS_CheckScope(CurrScope->Parent);
+	} /* esle */
+    } /* esle */
+
+} /* end of PRS_CheckGlobalScope */
+	
+
 /*** main functionality *************************************************/
 
 
@@ -421,7 +523,7 @@ sir_design *PRS_ParseDesign(		/* parse a whole design (#1) */
 	FILE		*FromFile,	/* from a file */
 	const char	*DesignName,	/* (returns a new SIR data structure */
 	const char	*FileName,	/*  or NULL and PRS_Error) */
-	GL_STRINGLIST	*ImportDirs /* = NULL */)
+	gl_stringlist	*ImportDirs /* = NULL */)
 {
 
 PRS_TheDesign = new SIR_Design(DesignName);	/* start with a new design */
@@ -431,6 +533,11 @@ PRS_lineno = 1;			/* current line */
 PRS_CurrentFile = FileName;	/* current filename */
 PRS_CurrFileInfo = PRS_TheDesign->FileList->FindOrInsert(FileName);
 PRS_ImportDirList = ImportDirs;	/* publish the dirs to search for import */
+PRS_FunctionBody = NULL;	/* not used in language mode */
+PRS_StoreFunctionBody = false;	/* not used in language mode */
+PRS_Constant = NULL;		/* not used */
+PRS_Annotation = NULL;		/* not used */
+PRS_Type = NULL;		/* not used */
 PRS_in = FromFile;		/* publish the file descriptor */
 PRS_Initialize(PRS_in,		/* and (re-)start the scanner  */
 		PRS_PARSE_LANGUAGE);
@@ -486,10 +593,20 @@ int main(int argc, char **argv)
 char			*FromName,
 			*ToName;
 FILE			*FromFile;
-GL_HANDLE		*ToFile;
+gl_io			*ToFile;
 sir_design		*NewDesign;
-static GL_STRINGLIST	ImportDir = { NULL, "." };
+static gl_stringlist	ImportDir = { NULL, "." };
 ERROR			Error;
+#ifdef DEBUG_SEQUENCE_OF_CALLS
+sir_symbol              *Behavior;
+sir_symbol              *MainMethod;
+sir_statement           *Body;
+sir_labels              *Labels;
+sir_constant            *Const;
+sir_initializer         *Annotation;
+sir_type                *Type;
+#endif /* DEBUG_SEQUENCE_OF_CALLS */
+
 
 printf("DEBUGGING: Parser ('%s')\n\n", argv[0]);
 
@@ -517,6 +634,190 @@ if (!(NewDesign = PRS_ParseDesign(FromFile, GL_GetBaseName(FromName),
 GL_CloseFile(FromFile);
 printf("Successfully parsed %d lines.\n", PRS_NumberOfLines);
 
+#ifdef DEBUG_SEQUENCE_OF_CALLS
+
+delete NewDesign;
+printf("Parsing '%s'...\n", FromName);
+if (!(FromFile = GL_OpenFileForReading(FromName)))
+   { GL_PrintError(GL_Error, GL_ErrorText(GL_Error));
+     exit(RESULT_ERROR);
+    } /* fi */
+if (!(NewDesign = PRS_ParseDesign(FromFile, GL_GetBaseName(FromName),
+					FromName, &ImportDir)))
+   { GL_PrintError(PRS_Error, PRS_ErrorText(PRS_Error));
+     fclose(FromFile);
+     exit(RESULT_ERROR);
+    } /* fi */
+GL_CloseFile(FromFile);
+printf("Successfully parsed %d lines.\n", PRS_NumberOfLines);
+
+Const = NULL;
+if ((Error = PRS_ParseConstant(
+		"\"This is a string const.\\n\"",
+		NewDesign, "InternalString", &Const)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed constant %s.\n", Const->Print());
+delete Const;
+
+Const = NULL;
+if ((Error = PRS_ParseConstant(
+		"/* embedded comment */ 42 ",
+		NewDesign, "InternalString2", &Const)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed constant %s.\n", Const->Print());
+delete Const;
+
+Annotation = NULL;
+if ((Error = PRS_ParseAnnotation(
+		"\"This is simple string annotation.\\n\"",
+		NewDesign, "InternalString3", &Annotation)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed annotation %s.\n", Annotation->Print());
+delete Annotation;
+
+Annotation = NULL;
+if ((Error = PRS_ParseAnnotation(
+		"{{1,2,3},{4,5,6},{7,8,9}}",
+		NewDesign, "InternalString4", &Annotation)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed annotation %s.\n", Annotation->Print());
+delete Annotation;
+
+Type = NULL;
+if ((Error = PRS_ParseType(
+		"const /* not volatile */ unsigned bit[7:0] // byte",
+		NewDesign, "InternalString", &Type)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed type %s.\n", Type->Print());
+
+Type = NULL;
+if ((Error = PRS_ParseType(
+		"struct NewStruct { int i, j; unsigned bit[32] b; }",
+		NewDesign, "InternalString", &Type)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed type %s.\n", Type->Print());
+
+Type = NULL;
+if ((Error = PRS_ParseType(
+		"signal bool",
+		NewDesign, "InternalString", &Type)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed type %s.\n", Type->Print());
+
+Type = NULL;
+if ((Error = PRS_ParseType(
+		"buffered bit[8][16]",
+		NewDesign, "InternalString", &Type)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed type %s.\n", Type->Print());
+
+Type = NULL;
+if ((Error = PRS_ParseType(
+		"in signal unsigned bit[1:8][2]",
+		NewDesign, "InternalString", &Type)))
+   { GL_PrintError(Error, PRS_ErrorText(Error));
+     delete NewDesign;
+     exit(RESULT_ERROR);
+    } /* fi */
+printf("Successfully parsed type %s.\n", Type->Print());
+
+if (  (Behavior = NewDesign->Symbols->Find("Main"))
+    &&(Behavior->ClassScope)
+    &&(MainMethod = Behavior->ClassScope->FindLocally("main"))
+    &&(MainMethod->IsFunctionDefinition()))
+   { puts("Parsing function from text string...");
+     if ((Error = PRS_ParseFunctionBody(NewDesign, MainMethod,
+					"{ /* test */\n"
+					"int x, y;\n"
+					"struct s { int a, b; } S;\n"
+					"note x.type = \"int\";\n"
+					"l1: x = 2 * y + S.a - S.b;\n"
+					"}\n",
+					"InternalFile",
+					&Body, &Labels)))
+	{ GL_PrintError(Error, PRS_ErrorText(Error));
+	  delete NewDesign;
+	  exit(RESULT_ERROR);
+	 } /* fi */
+     printf("Successfully parsed %d lines.\n", PRS_NumberOfLines);
+     assert(Body != NULL);
+     assert(Labels != NULL);
+     assert(Body->Scope != NULL);
+     Body->Scope->DFS_ForAllUserTypes(&SIR_UserType::DeleteYourType, NULL);
+     delete Body;	/* throw parsed body away */
+     delete Labels;
+
+     puts("Parsing function from text string...");
+     if ((Error = PRS_ParseFunctionBody(NewDesign, MainMethod,
+					"{ /* test */\n"
+					"int x, y, z;\n"
+					"note x.type = \"int\";\n"
+					"x = 2 * y ^ z;\n"
+					"}\n",
+					"InternalFile2",
+					&Body, &Labels)))
+	{ GL_PrintError(Error, PRS_ErrorText(Error));
+	  delete NewDesign;
+	  exit(RESULT_ERROR);
+	 } /* fi */
+     printf("Successfully parsed %d lines.\n", PRS_NumberOfLines);
+     NewDesign->DeleteLevel2();
+     assert(Body != NULL);
+     assert(Labels != NULL);
+     assert(MainMethod->FctBody->Scope != NULL);
+     MainMethod->FctBody->Scope->DFS_ForAllUserTypes(
+				&SIR_UserType::DeleteYourType, NULL);
+     delete MainMethod->FctBody;
+     delete MainMethod->Labels;
+     MainMethod->FctBody = Body;	/* transplant parsed body */
+     MainMethod->Labels = Labels;
+     NewDesign->BuildLevel2();
+     puts("Successfully transplanted function body.");
+    } /* fi */
+
+#ifndef DEBUG_OUTPUT_MODIFIED_DESIGN
+delete NewDesign;
+printf("Parsing '%s'...\n", FromName);
+if (!(FromFile = GL_OpenFileForReading(FromName)))
+   { GL_PrintError(GL_Error, GL_ErrorText(GL_Error));
+     exit(RESULT_ERROR);
+    } /* fi */
+if (!(NewDesign = PRS_ParseDesign(FromFile, GL_GetBaseName(FromName),
+					FromName, &ImportDir)))
+   { GL_PrintError(PRS_Error, PRS_ErrorText(PRS_Error));
+     fclose(FromFile);
+     exit(RESULT_ERROR);
+    } /* fi */
+GL_CloseFile(FromFile);
+printf("Successfully parsed %d lines.\n", PRS_NumberOfLines);
+#endif /* DEBUG_OUTPUT_MODIFIED_DESIGN */
+
+#endif /* DEBUG_SEQUENCE_OF_CALLS */
+
 printf("\nDumping '%s'...\n", ToName);
 if (!(ToFile = GL_IO::OpenFile(ToName, GL_IO_WRITE)))
    { GL_PrintError(PRS_Error, PRS_ErrorText(PRS_Error));
