@@ -2,7 +2,7 @@
 // longlong.cc: "long long" data type, signed & unsigned
 // ============================================================================
 // 
-//
+// 10/09/04 <gerstl>	added typedefs for ___longlong_rep
 // 11/21/01 <gerstl>	Cleanup of test case
 // 05/24/01 <gerstl>	Initial version
 //
@@ -123,7 +123,7 @@ long long str2ll(unsigned int base, const char *str)
 {
   return str2ull(base, str);
 }
- 
+
 #else // #ifdef HAVE_LLONG
 
 
@@ -131,13 +131,8 @@ long long str2ll(unsigned int base, const char *str)
 /* Non-native _longlong support */
 
 
-// Number of bits in a "long"
-#ifndef LONG_BIT
-#define LONG_BIT  	(sizeof(long) * CHAR_BIT)
-#endif
-
-// Number of bits in one half of a "long"
-#define HALFLONG_BIT	(LONG_BIT >> 1)
+// Number of bits in one half of a chunk
+#define HALFLLREP_BIT	(LLREP_BIT >> 1)
 
 
 
@@ -197,8 +192,8 @@ _longlong<false> str2ll(unsigned int base, const char *str) {
 // Double, Long Double
 __longlong& __longlong::operator= (double d)
 {
-   s1 = (long)(d / ((double)ULONG_MAX + 1));
-   d -= (double)s1 * ((double)ULONG_MAX + 1);
+   s1 = (long)(d / ((double)ULLREP_MAX + 1));
+   d -= (double)s1 * ((double)ULLREP_MAX + 1);
    u0 = (unsigned long)d;
    
    return *this;
@@ -206,8 +201,8 @@ __longlong& __longlong::operator= (double d)
 
 __longlong& __longlong::operator= (long double d)
 {
-   s1 = (long)(d / ((long double)ULONG_MAX + 1));
-   d -= (long double)s1 * ((long double)ULONG_MAX + 1);
+   s1 = (long)(d / ((long double)ULLREP_MAX + 1));
+   d -= (long double)s1 * ((long double)ULLREP_MAX + 1);
    u0 = (unsigned long)d;
    
    return *this;
@@ -278,22 +273,22 @@ void __longlong::set(const char* str, unsigned int base)
 /* --- Multiplication --- */
 
 void __longlong::mul(const __longlong& ll) {
-  unsigned long tmp;
+  ___longlong_urep tmp;
   int carry = 0;
   
   // Split u0 and ll.u0 into halves
-  unsigned long a0 = u0    & (ULONG_MAX >> HALFLONG_BIT);
-  unsigned long b0 = ll.u0 & (ULONG_MAX >> HALFLONG_BIT);
-  unsigned long a1 = u0    >> HALFLONG_BIT;
-  unsigned long b1 = ll.u0 >> HALFLONG_BIT;
+  ___longlong_urep a0 = u0    & (ULLREP_MAX >> HALFLLREP_BIT);
+  ___longlong_urep b0 = ll.u0 & (ULLREP_MAX >> HALFLLREP_BIT);
+  ___longlong_urep a1 = u0    >> HALFLLREP_BIT;
+  ___longlong_urep b1 = ll.u0 >> HALFLLREP_BIT;
     
   // Calculate overflow of lower chunk into upper chunk
   tmp = a0 * b0;
-  tmp >>= HALFLONG_BIT;
+  tmp >>= HALFLLREP_BIT;
   tmp += a0 * b1;  // carry += (tmp < (a0*b1));  
   tmp += a1 * b0;  carry += (tmp < (a1*b0));
-  tmp >>= HALFLONG_BIT;
-  tmp += ((unsigned long)carry << HALFLONG_BIT);
+  tmp >>= HALFLLREP_BIT;
+  tmp += ((unsigned long)carry << HALFLLREP_BIT);
   tmp += a1 * b1;
   
   // Calculate upper chunk
@@ -310,21 +305,21 @@ void __longlong::mul(const __longlong& ll) {
 //            Volume 2, Seminumerical Algorithms
 
 
-#define NORM(hi,lo,sh)		(((hi)<<(sh)) | ((lo)>>(HALFLONG_BIT-(sh))))
-#define MNORM(hi,lo,sh)		((NORM(hi,lo,sh)) & HALFLONG_BIT)
+#define NORM(hi,lo,sh)		(((hi)<<(sh)) | ((lo)>>(HALFLLREP_BIT-(sh))))
+#define MNORM(hi,lo,sh)		((NORM(hi,lo,sh)) & HALFLLREP_BIT)
 
 
 // Calculate quotient (q is estimated from n/d)
-unsigned long __longlong::calc_q( unsigned long n1, unsigned long n2,
-				  unsigned long d1, unsigned long d2 )
+___longlong_urep __longlong::calc_q( ___longlong_urep n1, ___longlong_urep n2,
+				     ___longlong_urep d1, ___longlong_urep d2 )
 {
-  unsigned long q, r;
+  ___longlong_urep q, r;
   
   q = n1 / d1;
   r = n1 % d1;
   
   // Adjust
-  while( q * d2 > (r << HALFLONG_BIT) + n2) {  
+  while( q * d2 > (r << HALFLLREP_BIT) + n2) {
     q--;
     r += d1;
     if( r < d1 ) break;  // overflow
@@ -334,31 +329,31 @@ unsigned long __longlong::calc_q( unsigned long n1, unsigned long n2,
 
   
 // Multiplty & subtract (*this -= q * d)
-void __longlong::mul_sub( unsigned long& q,
-			  unsigned long d1, unsigned long d2, 
-			  unsigned long d3, unsigned long d4 )
+void __longlong::mul_sub( ___longlong_urep& q,
+			  ___longlong_urep d1, ___longlong_urep d2, 
+			  ___longlong_urep d3, ___longlong_urep d4 )
 {
-  unsigned long sav0 = u0;
-  unsigned long sav1 = u1;
-  unsigned long mul;
+  ___longlong_urep sav0 = u0;
+  ___longlong_urep sav1 = u1;
+  ___longlong_urep mul;
   
   // Multiply & subtract
   mul   = q * d4;
-  mul >>= HALFLONG_BIT;
+  mul >>= HALFLLREP_BIT;
   mul  += q * d3;
-  mul >>= HALFLONG_BIT;
+  mul >>= HALFLLREP_BIT;
   mul  += q * d2;  
   
-  u0   -= q * d4 + ((q * d3) << HALFLONG_BIT);
-  u1   -= mul + ((q * d1) << HALFLONG_BIT) + (u0 > sav0);
+  u0   -= q * d4 + ((q * d3) << HALFLLREP_BIT);
+  u1   -= mul + ((q * d1) << HALFLLREP_BIT) + (u0 > sav0);
       
   // Test remainder
-  if( (u1 > sav1) || ((q * d1) > (ULONG_MAX>>HALFLONG_BIT)) ) {
+  if( (u1 > sav1) || ((q * d1) > (ULLREP_MAX>>HALFLLREP_BIT)) ) {
     // Adjust, add back
     q--;
     sav0 = u0;
-    u0 += d4 + (d3 << HALFLONG_BIT);
-    u1 += d2 + (d1 << HALFLONG_BIT) + (u0 < sav0);
+    u0 += d4 + (d3 << HALFLLREP_BIT);
+    u1 += d2 + (d1 << HALFLLREP_BIT) + (u0 < sav0);
   }
 }
 
@@ -368,29 +363,29 @@ void __longlong::mul_sub( unsigned long& q,
 // quotient in the return value and the upper chunk.
 // How much of in-place data is remainder and how much is quotient 
 // depends on the size of the divisor (remainder <= divisor).
-unsigned long __longlong::div( const __longlong& ll )
+___longlong_urep __longlong::div( const __longlong& ll )
 {
   int norm = 0;
   
   // Split the denominator into chunks
-  unsigned long d1 = ll.u1 >> HALFLONG_BIT;
-  unsigned long d2 = ll.u1 & (ULONG_MAX >> HALFLONG_BIT);
-  unsigned long d3 = ll.u0 >> HALFLONG_BIT;
-  unsigned long d4 = ll.u0 & (ULONG_MAX >> HALFLONG_BIT);
+  ___longlong_urep d1 = ll.u1 >> HALFLLREP_BIT;
+  ___longlong_urep d2 = ll.u1 & (ULLREP_MAX >> HALFLLREP_BIT);
+  ___longlong_urep d3 = ll.u0 >> HALFLLREP_BIT;
+  ___longlong_urep d4 = ll.u0 & (ULLREP_MAX >> HALFLLREP_BIT);
   
   // Split the nominator into chunks
-  unsigned long n1 = u1    >> HALFLONG_BIT;
-  unsigned long n2 = u1    & (ULONG_MAX >> HALFLONG_BIT);
-  unsigned long n3 = u0    >> HALFLONG_BIT;
+  ___longlong_urep n1 = u1    >> HALFLLREP_BIT;
+  ___longlong_urep n2 = u1    & (ULLREP_MAX >> HALFLLREP_BIT);
+  ___longlong_urep n3 = u0    >> HALFLLREP_BIT;
   
-  unsigned long dhi, dlo;
-  unsigned long q, q2, q3;
+  ___longlong_urep dhi, dlo;
+  ___longlong_urep q, q2, q3;
   
   // Check size of divisor
   if( d1 )
   {
     // Determine normalization
-    while( !( (ULONG_MAX << (HALFLONG_BIT-norm-1)) & d1 ) ) norm++;
+    while( !( (ULLREP_MAX << (HALFLLREP_BIT-norm-1)) & d1 ) ) norm++;
     
     // Normalize divisor
     dhi = NORM(d1,d2,norm);
@@ -403,7 +398,7 @@ unsigned long __longlong::div( const __longlong& ll )
   else if( d2 )
   {
     // Determine normalization
-    while( !( (ULONG_MAX << (HALFLONG_BIT-norm-1)) & d2 ) ) norm++;
+    while( !( (ULLREP_MAX << (HALFLLREP_BIT-norm-1)) & d2 ) ) norm++;
     
     // Normalize divisor
     dhi = NORM(d2,d3,norm);
@@ -413,17 +408,17 @@ unsigned long __longlong::div( const __longlong& ll )
     mul_sub( q, d2, d3, d4, 0 );
     
     n1 = u1;
-    n2 = u0 >> HALFLONG_BIT;
-    n3 = u0 & (ULONG_MAX >> HALFLONG_BIT);
+    n2 = u0 >> HALFLLREP_BIT;
+    n3 = u0 & (ULLREP_MAX >> HALFLLREP_BIT);
     q2 = calc_q( NORM(n1,n2,norm), MNORM(n2,n3,norm), dhi, dlo );
     mul_sub(q2, 0, d2, d3, d4);
     
-    return (q << HALFLONG_BIT) + q2;
+    return (q << HALFLLREP_BIT) + q2;
   }
   else if( d3 )
   {
     // Determine normalization
-    while( !( (ULONG_MAX << (HALFLONG_BIT-norm-1)) & d3 ) ) norm++;
+    while( !( (ULLREP_MAX << (HALFLLREP_BIT-norm-1)) & d3 ) ) norm++;
     
     // Normalize divisor
     dhi = NORM(d3,d4,norm);
@@ -433,32 +428,32 @@ unsigned long __longlong::div( const __longlong& ll )
     mul_sub( q, d3, d4, 0, 0 );
     
     n1 = u1;
-    n2 = u0 >> HALFLONG_BIT;
-    n3 = u0 & (ULONG_MAX >> HALFLONG_BIT);
+    n2 = u0 >> HALFLLREP_BIT;
+    n3 = u0 & (ULLREP_MAX >> HALFLLREP_BIT);
     q2 = calc_q( NORM(n1,n2,norm), MNORM(n2,n3,norm), dhi, dlo );
     mul_sub(q2, 0, d3, d4, 0 );
         
-    n1 = (u1 << HALFLONG_BIT) + (u0 >> HALFLONG_BIT);
-    n2 = u0 & (ULONG_MAX >> HALFLONG_BIT);
+    n1 = (u1 << HALFLLREP_BIT) + (u0 >> HALFLLREP_BIT);
+    n2 = u0 & (ULLREP_MAX >> HALFLLREP_BIT);
     q3 = calc_q( NORM(n1,n2,norm), MNORM(n2, 0,norm), dhi, dlo );
     mul_sub(q3, 0, 0, d3, d4 );
     
     u1 = q;
-    return (q2 << HALFLONG_BIT) + q3;
+    return (q2 << HALFLLREP_BIT) + q3;
   }
   
   // Single-chunk divisor
   q = n1 / d4;
-  n1 = ((n1 % d4) << HALFLONG_BIT) + n2;
+  n1 = ((n1 % d4) << HALFLLREP_BIT) + n2;
   q2 = n1 / d4;
-  n1 = ((n1 % d4) << HALFLONG_BIT) + (u0 >> HALFLONG_BIT);
+  n1 = ((n1 % d4) << HALFLLREP_BIT) + (u0 >> HALFLLREP_BIT);
   q3 = n1 / d4;
-  n1 = ((n1 % d4) << HALFLONG_BIT) + (u0 & (ULONG_MAX >> HALFLONG_BIT));
+  n1 = ((n1 % d4) << HALFLLREP_BIT) + (u0 & (ULLREP_MAX >> HALFLLREP_BIT));
 
   u0 = n1 % d4;
   
-  u1 = (q << HALFLONG_BIT) + q2;
-  return (q3 << HALFLONG_BIT) + (n1 / d4);
+  u1 = (q << HALFLLREP_BIT) + q2;
+  return (q3 << HALFLLREP_BIT) + (n1 / d4);
 }
 
 #endif  // #ifdef HAVE_LLONG
@@ -798,9 +793,9 @@ int main(int argc, char** argv)
 
     
   // Test operations
-  for(i = 0, au = 0, xu = 0; i <= LONG_BIT*2; au = (au<<1) + ++i%2, xu = (xu<<1) + i%2)
+  for(i = 0, au = 0, xu = 0; i <= LLREP_BIT*2; au = (au<<1) + ++i%2, xu = (xu<<1) + i%2)
   {
-    for(j = 0, bu = 0, yu = 0; j <= LONG_BIT*2; bu = (bu<<1) + ++j%2, yu = (yu<<1) + j%2) {
+    for(j = 0, bu = 0, yu = 0; j <= LLREP_BIT*2; bu = (bu<<1) + ++j%2, yu = (yu<<1) + j%2) {
       testOp(au, bu, xu, yu);
       testOpShadow(au, bu, xu, yu);
       testOpMix(au, bu, xu, yu);
@@ -811,7 +806,7 @@ int main(int argc, char** argv)
     x = (_longlong<>)xu;
         
     // shifting
-    for(unsigned int k = 0; k < LONG_BIT*2; k++)
+    for(unsigned int k = 0; k < LLREP_BIT*2; k++)
     {
       c = a << k;  z = x << k;
       if( c != z.toLLong() ) operror("<<", a, k, c, z);
@@ -823,9 +818,9 @@ int main(int argc, char** argv)
       if( cu != zu.toULLong() ) operror(">>", au, k, cu, zu);
     }
   }
-  for(i = 0, au = 0, xu = 0; i <= LONG_BIT*2; i++, au = (au<<1) + 1, xu = (xu<<1) + 1)
+  for(i = 0, au = 0, xu = 0; i <= LLREP_BIT*2; i++, au = (au<<1) + 1, xu = (xu<<1) + 1)
   {
-    for(j = 0, bu = 0, yu = 0; j <= LONG_BIT*2; j++, bu = (bu<<1) + 1, yu = (yu<<1) + 1) {
+    for(j = 0, bu = 0, yu = 0; j <= LLREP_BIT*2; j++, bu = (bu<<1) + 1, yu = (yu<<1) + 1) {
       testOp(au, bu, xu, yu);
       testOpShadow(au, bu, xu, yu);
       testOpMix(au, bu, xu, yu);
@@ -833,9 +828,9 @@ int main(int argc, char** argv)
   }
 
   // Now with shadow class
-  for(i = 0, au = 0, xsu = 0; i <= LONG_BIT*2; au = (au<<1) + ++i%2, xsu = (xsu<<1) + i%2)
+  for(i = 0, au = 0, xsu = 0; i <= LLREP_BIT*2; au = (au<<1) + ++i%2, xsu = (xsu<<1) + i%2)
   {
-    for(j = 0, bu = 0, ysu = 0; j <= LONG_BIT*2; bu = (bu<<1) + ++j%2, ysu = (ysu<<1) + j%2) {
+    for(j = 0, bu = 0, ysu = 0; j <= LLREP_BIT*2; bu = (bu<<1) + ++j%2, ysu = (ysu<<1) + j%2) {
       testOp(au, bu, xsu, ysu);
       testOpShadow(au, bu, xsu, ysu);
       testOpMix(au, bu, xsu, ysu);
@@ -846,7 +841,7 @@ int main(int argc, char** argv)
     xs = (_longlong<>)xsu;
         
     // shifting
-    for(unsigned int k = 0; k < LONG_BIT*2; k++)
+    for(unsigned int k = 0; k < LLREP_BIT*2; k++)
     {
       c = a << k;  zs = xs << k;
       if( c != zs.toLLong() ) operror("<<", a, k, c, zs);
@@ -858,9 +853,9 @@ int main(int argc, char** argv)
       if( cu != zsu.toULLong() ) operror(">>", au, k, cu, zsu);
     }
   }
-  for(i = 0, au = 0, xsu = 0; i <= LONG_BIT*2; i++, au = (au<<1) + 1, xsu = (xsu<<1) + 1)
+  for(i = 0, au = 0, xsu = 0; i <= LLREP_BIT*2; i++, au = (au<<1) + 1, xsu = (xsu<<1) + 1)
   {
-    for(j = 0, bu = 0, ysu = 0; j <= LONG_BIT*2; j++, bu = (bu<<1) + 1, ysu = (ysu<<1) + 1) {
+    for(j = 0, bu = 0, ysu = 0; j <= LLREP_BIT*2; j++, bu = (bu<<1) + 1, ysu = (ysu<<1) + 1) {
       testOp(au, bu, xsu, ysu);
       testOpShadow(au, bu, xsu, ysu);
       testOpMix(au, bu, xsu, ysu);
