@@ -2,6 +2,8 @@
 // longlong.h: "long long" data type support
 // ============================================================================
 // 
+// 10/09/04 <gerstl>	added typedefs for ___longlong_rep
+// 04/08/02 <doemer>	added typedefs _LONGLONG and _ULONGLONG
 // 11/28/01 <gerstl>	Removed ambiguities, move more casting into scc
 // 05/24/01 <gerstl>	Initial version
 //
@@ -29,7 +31,7 @@ _CPP_include <limits.h>
 _CPP_undef  HAVE_LLONG  
 _CPP_define HAVE_LLONG  
 #define HAVE_LLONG
-  
+
 // Provide constants
 _CPP_ifndef LONG_LONG_MAX
 _CPP_ifdef LLONG_MAX
@@ -60,6 +62,12 @@ _CPP_define ULLONG_MAX        ULONG_LONG_MAX
 _CPP_endif
 #endif
   
+
+/* --- Provide typedefs --- */
+
+typedef long long             _LONGLONG;
+typedef unsigned long long    _ULONGLONG;
+
   
 /* ---  Global functions --- */
   
@@ -89,16 +97,18 @@ _CPP_define  _NO_LONGLONG
 #define _NO_LONGLONG
 #endif
 
+
 // Non-native long long version of library    
 _CPP_undef  HAVE_LLONG        
 
   
 _CPP_include <limits.h>
     
+// Provide constants
 _CPP_undef  LONG_LONG_MAX
-_CPP_define LONG_LONG_MAX    (_longlong<false>(LONG_MAX, ULONG_MAX))
+_CPP_define LONG_LONG_MAX    (_longlong<false>(LLREP_MAX, ULLREP_MAX))
 _CPP_undef  ULONG_LONG_MAX
-_CPP_define ULONG_LONG_MAX   (_longlong<true>(ULONG_MAX, ULONG_MAX))
+_CPP_define ULONG_LONG_MAX   (_longlong<true>(ULLREP_MAX, ULLREP_MAX))
     
 _CPP_undef  LLONG_MAX    
 _CPP_define LLONG_MAX         LONG_LONG_MAX
@@ -116,6 +126,12 @@ template<bool> class _longlong;
 template<bool> class _longlong_;
 
 
+/* --- Provide typedefs --- */
+
+typedef _longlong<false>      _LONGLONG;
+typedef _longlong<true>       _ULONGLONG;
+
+
 /* --- Global functions --- */
 
 extern "C" {
@@ -134,7 +150,44 @@ _longlong<true> str2ull(unsigned int base, const char* str);
 /* ------------------------------------------------------------------------- */
 /* Class for type "long long", signed/unsigned */
 
+#include "sizing.h"
 
+/* --- internal representation; if this changes, adapt limits.in.sh ! --- */
+
+#ifdef HAVE_INT_LONG
+
+typedef int		___longlong_rep;	
+typedef unsigned int	___longlong_urep;
+typedef signed int	___longlong_srep;
+
+#ifdef FIRST_PASS
+_CPP_define LLREP_BIT	(sizeof(___longlong_rep) * CHAR_BIT)
+_CPP_define ULLREP_MAX	UINT_MAX
+_CPP_define LLREP_MAX	INT_MAX
+#else
+#define LLREP_BIT	(sizeof(___longlong_rep) * CHAR_BIT)
+#define ULLREP_MAX	UINT_MAX
+#define LLREP_MAX	INT_MAX
+#endif
+
+#else
+
+typedef long		___longlong_rep;	
+typedef unsigned long	___longlong_urep;
+typedef signed long	___longlong_srep;
+
+#ifdef FIRST_PASS
+_CPP_define LLREP_BIT	(sizeof(___longlong_rep) * CHAR_BIT)
+_CPP_define ULLREP_MAX	ULONG_MAX
+_CPP_define LLREP_MAX	LONG_MAX
+#else
+#define LLREP_BIT	(sizeof(___longlong_rep) * CHAR_BIT)
+#define ULLREP_MAX	ULONG_MAX
+#define LLREP_MAX	LONG_MAX
+#endif
+
+#endif  
+  
 /* --- Base class, long long representation --- */
 
 class ___longlong
@@ -142,12 +195,12 @@ class ___longlong
 protected:
   // long long representation
   union {
-    unsigned long u0;
-    signed   long s0;
+    ___longlong_urep u0;
+    ___longlong_srep s0;
   };
   union {
-    unsigned long u1;
-    signed   long s1;
+    ___longlong_urep u1;
+    ___longlong_srep s1;
   };
   
 public:  
@@ -168,10 +221,10 @@ public:
 #ifdef DEBUG
 #if defined(LLONG_MAX) || defined(LONG_LONG_MAX) || defined(QUAD_MAX)
   unsigned long long toULLong() const { 
-    return (unsigned long long)u0 + ((unsigned long long)u1 << sizeof(long)*CHAR_BIT);
+    return (unsigned long long)u0 + ((unsigned long long)u1 << LLREP_BIT);
   }
   long long toLLong() const { 
-    return (long long)u0 + ((long long)s1 << sizeof(long)*CHAR_BIT);
+    return (long long)u0 + ((long long)s1 << LLREP_BIT);
   }
 #endif
 #endif
@@ -185,7 +238,7 @@ class __longlong: public ___longlong
 public:
   // Initialization / Construction
   __longlong()                                                             { }
-  __longlong(unsigned long hi, unsigned long lo)         { u1 = hi; u0 = lo; }
+  __longlong(___longlong_urep hi, ___longlong_urep lo)   { u1 = hi; u0 = lo; }
   __longlong(const char* s, unsigned int base = 0)	     { set(s, base); }
   __longlong(int i)                                             { *this = i; }
   __longlong(unsigned int i)                                    { *this = i; }
@@ -245,12 +298,12 @@ protected:
   void          mul( const __longlong& ll );
   
   // division helpers
-  unsigned long div( const __longlong& ll );
-  unsigned long calc_q( unsigned long n1, unsigned long n2,
-		        unsigned long d1, unsigned long d2 );
-  void          mul_sub( unsigned long& q,
-			 unsigned long d1, unsigned long d2, 
-			 unsigned long d3, unsigned long d4 );
+  ___longlong_urep div( const __longlong& ll );
+  ___longlong_urep calc_q( ___longlong_urep n1, ___longlong_urep n2,
+		           ___longlong_urep d1, ___longlong_urep d2 );
+  void             mul_sub( ___longlong_urep& q,
+			    ___longlong_urep d1, ___longlong_urep d2, 
+			    ___longlong_urep d3, ___longlong_urep d4 );
 };
 
 
@@ -262,7 +315,7 @@ template<bool usign=false> class _longlong : public __longlong
 public:  
   // Initialization / Construction
   _longlong()                                    : __longlong()         { }
-  _longlong(unsigned long hi, unsigned long lo)  : __longlong(hi, lo)   { }
+  _longlong(___longlong_urep hi, ___longlong_urep lo):__longlong(hi, lo){ }
   _longlong(const char* s, unsigned int base = 0): __longlong(s, base)  { }
   _longlong(const __longlong& ll)                : __longlong(ll)       { }
   _longlong(int i)                               : __longlong(i)        { }
@@ -752,13 +805,13 @@ template<bool usign>
 // Left shift
 inline __longlong& __longlong::operator<<=(unsigned int sh)
 {
-  if( sh >= (sizeof(long)*CHAR_BIT) ) {
-    u1 = u0 << (sh - (sizeof(long)*CHAR_BIT));
+  if( sh >= LLREP_BIT ) {
+    u1 = u0 << (sh - LLREP_BIT);
     u0 = 0;
   } 
   else {
     u1 <<= sh;
-    if( sh ) u1 |= (u0 >> (sizeof(long)*CHAR_BIT - sh));
+    if( sh ) u1 |= (u0 >> (LLREP_BIT - sh));
     u0 <<= sh;
   }
   return *this;
@@ -768,16 +821,16 @@ inline __longlong& __longlong::operator<<=(unsigned int sh)
 template<bool usign>
   inline _longlong<usign>& _longlong<usign>::operator>>=(unsigned int sh) 
 {
-  if( sh >= (sizeof(long)*CHAR_BIT) ) {
+  if( sh >= LLREP_BIT ) {
     if( usign ) 
-      u0 = u1 >> (sh - sizeof(long)*CHAR_BIT);
+      u0 = u1 >> (sh - LLREP_BIT);
     else
-      s0 = s1 >> (sh - sizeof(long)*CHAR_BIT);
-    s1 = usign? 0 : s1 >>= (sizeof(long)*CHAR_BIT - 1);
+      s0 = s1 >> (sh - LLREP_BIT);
+    s1 = usign? 0 : s1 >>= (LLREP_BIT - 1);
   }
   else {
     u0 >>= sh;
-    if( sh ) u0 |= (u1 << (sizeof(long)*CHAR_BIT - sh));
+    if( sh ) u0 |= (u1 << (LLREP_BIT - sh));
     if( usign ) u1 >>= sh; else s1 >>= sh;
   }
   return *this;
@@ -813,7 +866,6 @@ template<bool usign>
   return ll;
 }
 
-
 #endif
 
 #ifdef FIRST_PASS
